<?php
/************************************************************************
 * This file is part of EspoCRM.
 *
 * EspoCRM - Open Source CRM application.
 * Copyright (C) 2014-2020 Yuri Kuznetsov, Taras Machyshyn, Oleksiy Avramenko
 * Website: https://www.espocrm.com
 *
 * EspoCRM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * EspoCRM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with EspoCRM. If not, see http://www.gnu.org/licenses/.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU General Public License version 3,
 * these Appropriate Legal Notices must retain the display of the "EspoCRM" word.
 ************************************************************************/

namespace Espo\ORM;

use Espo\ORM\{
    QueryParams\SelectBuilder,
    QueryParams\UpdateBuilder,
    QueryParams\DeleteBuilder,
    QueryParams\InsertBuilder,
    QueryParams\UnionBuilder,
    QueryParams\Query,
    QueryParams\Builder,
};

use ReflectionClass;
use RuntimeException;

/**
 * Creates query builders for specific query types.
 */
class QueryBuilder
{
    /**
     * Proceed with SELECT builder.
     */
    public function select() : SelectBuilder
    {
        return new SelectBuilder();
    }

    /**
     * Proceed with UPDATE builder.
     */
    public function update() : UpdateBuilder
    {
        return new UpdateBuilder();
    }

    /**
     * Proceed with DELETE builder.
     */
    public function delete() : DeleteBuilder
    {
        return new DeleteBuilder();
    }

    /**
     * Proceed with INSERT builder.
     */
    public function insert() : InsertBuilder
    {
        return new InsertBuilder();
    }

    /**
     * Proceed with UNION builder.
     */
    public function union() : UnionBuilder
    {
        return new UnionBuilder();
    }

    /**
     * Clone an existing query and proceed modifying it.
     */
    public function clone(Query $query) : Builder
    {
        $class = new ReflectionClass($query);

        $methodName = ucfirst($class->getShortName());

        if (!method_exists($this, $methodName)) {
            throw new RuntimeException("Can't clone an unsupported query.");
        }

        return $this->$methodName()->clone($query);
    }
}
